﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;
using SushiBarDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SushiBarDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        [Required]
        public int ClientId { get; private set; }
        public virtual Client Client { get; private set; } = new();
        [Required]
        public int SushiId { get; private set; }

        public virtual Sushi Sushi { get; set; } = new();

        [Required]
        public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public static Order Create(SushiBarDatabase context, OrderBindingModel model)
        {
            return new Order()
            {
                Id = model.Id,
                ClientId = model.ClientId,
                Client = context.Clients.First(x => x.Id == model.ClientId),
                SushiId = model.SushiId,
                Sushi = context.Sushis.First(x => x.Id == model.SushiId),
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            SushiId = SushiId,
            SushiName = Sushi.SushiName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
        };
    }
}
