﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace SushiBarFileImplement.Models
{
    public class Sushi : ISushiModel
    {
        public int Id { get; private set; }
        public string SushiName { get; private set; } = string.Empty;
        public double Price { get; private set; }
        public Dictionary<int, int> Components { get; private set; } = new();
        private Dictionary<int, (IComponentModel, int)>? _SushiComponents = null;
        public Dictionary<int, (IComponentModel, int)> SushiComponents
        {
            get
            {
                if (_SushiComponents == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _SushiComponents = Components.ToDictionary(x => x.Key, y => ((source.Components.FirstOrDefault(z => z.Id == y.Key) as IComponentModel)!, y.Value));
                }
                return _SushiComponents;
            }
        }
        public static Sushi? Create(SushiBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Sushi()
            {
                Id = model.Id,
                SushiName = model.SushiName,
                Price = model.Price,
                Components = model.SushiComponents.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Sushi? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Sushi()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                SushiName = element.Element("SushiName")!.Value,
                Price = Convert.ToDouble(element.Element("Price")!.Value),
                Components = element.Element("SushiComponents")!.Elements("SushiComponent").ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value),
                x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(SushiBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            SushiName = model.SushiName;
            Price = model.Price;
            Components = model.SushiComponents.ToDictionary(x => x.Key, x => x.Value.Item2);
            _SushiComponents = null;
        }
        public SushiViewModel GetViewModel => new()
        {
            Id = Id,
            SushiName = SushiName,
            Price = Price,
            SushiComponents = SushiComponents
        };
        public XElement GetXElement => new("Sushi",
                new XAttribute("Id", Id),
                new XElement("SushiName", SushiName),
                new XElement("Price", Price.ToString()),
                new XElement("SushiComponents", Components.Select(
                    x => new XElement("SushiComponent", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()));
    }
}
