using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using SushiBarContracts.BusinessLogicsContracts;
using System;
using SushiBarBusinessLogic.BusinessLogics;
using SushiBarDatabaseImplement.Implements;
using SushiBarContracts.StoragesContracts;
using SushiBarView;
using NLog.Extensions.Logging;
using SushiBarBusinessLogic.OfficePackage.Implements;
using SushiBarBusinessLogic.OfficePackage;
using SushiBarView.Reports;
using SushiBar.StoragesContracts;
using Microsoft.EntityFrameworkCore.Design;
using SushiBar.BusinessLogicsContracts;
using NLog.Extensions.Logging;
using SushiBarView;

namespace SushiBarView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<ISushiStorage, SushiStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();

            services.AddTransient<IShopStorage, ShopStorage>();
            services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<ISushiLogic, SushiLogic>();
            services.AddTransient<IClientLogic, ClientLogic>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormComponent>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormSushis>();
            services.AddTransient<FormSushiComponent>();
            services.AddTransient<FormSushi>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportSushiComponents>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<FormClients>();
            services.AddTransient<EntityFrameworkDesignServicesBuilder>();
            services.AddTransient<IShopLogic, ShopLogic>();

            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormMain>();
            services.AddTransient<IShopStorage, ShopStorage>();
            services.AddTransient<IShopLogic, ShopLogic>();
            services.AddTransient<FormShop>();
            services.AddTransient<FormShops>();
            services.AddTransient<FormCreateSupply>();
            services.AddTransient<FormSellSushi>();
            services.AddTransient<FormReportSushiComponents>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportShop>();
            services.AddTransient<FormReportGroupedOrders>();
        }
    }
}